@@ -2,12 +2,11 @@
 # std imports
 import platform
 
-# 3rd party
-import six
-
 # local
+from blessed._compat import TextType, StringType
 from blessed.colorspace import CGA_COLORS, X11_COLORNAMES_TO_RGB
 
+# isort: off
 # curses
 if platform.system() == 'Windows':
     import jinxed as curses   # pylint: disable=import-error
@@ -20,6 +19,7 @@ def _make_colors():
     Return set of valid colors and their derivatives.
 
     :rtype: set
+    :returns: Color names with prefixes
     """
     colors = set()
     # basic CGA foreground color, background, high intensity, and bold
@@ -46,30 +46,31 @@ COLORS = _make_colors()
 COMPOUNDABLES = set('bold underline reverse blink italic standout'.split())
 
 
-class ParameterizingString(six.text_type):
+class ParameterizingString(TextType):
     r"""
     A Unicode string which can be called as a parameterizing termcap.
 
     For example::
 
+        >>> from blessed import Terminal
         >>> term = Terminal()
         >>> color = ParameterizingString(term.color, term.normal, 'color')
         >>> color(9)('color #9')
         u'\x1b[91mcolor #9\x1b(B\x1b[m'
     """
 
-    def __new__(cls, *args):
+    def __new__(cls, cap, normal=u'', name=u'<not specified>'):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
         """
         Class constructor accepting 3 positional arguments.
 
-        :arg cap: parameterized string suitable for curses.tparm()
-        :arg normal: terminating sequence for this capability (optional).
-        :arg name: name of this terminal capability (optional).
+        :arg str cap: parameterized string suitable for curses.tparm()
+        :arg str normal: terminating sequence for this capability (optional).
+        :arg str name: name of this terminal capability (optional).
         """
-        assert args and len(args) < 4, args
-        new = six.text_type.__new__(cls, args[0])
-        new._normal = args[1] if len(args) > 1 else u''
-        new._name = args[2] if len(args) > 2 else u'<not specified>'
+        new = TextType.__new__(cls, cap)
+        new._normal = normal
+        new._name = name
         return new
 
     def __call__(self, *args):
@@ -80,7 +81,10 @@ class ParameterizingString(six.text_type):
         ``*args``, followed by the terminating sequence (self.normal) into
         a :class:`FormattingString` capable of being called.
 
+        :raises TypeError: Mismatch between capability and arguments
+        :raises curses.error: :func:`curses.tparm` raised an exception
         :rtype: :class:`FormattingString` or :class:`NullCallableString`
+        :returns: Callable string for given parameters
         """
         try:
             # Re-encode the cap, because tparm() takes a bytestring in Python
@@ -91,12 +95,10 @@ class ParameterizingString(six.text_type):
         except TypeError as err:
             # If the first non-int (i.e. incorrect) arg was a string, suggest
             # something intelligent:
-            if args and isinstance(args[0], six.string_types):
+            if args and isinstance(args[0], StringType):
                 raise TypeError(
-                    "A native or nonexistent capability template, %r received"
-                    " invalid argument %r: %s. You probably misspelled a"
-                    " formatting call like `bright_red'" % (
-                        self._name, args, err))
+                    "Unknown terminal capability, %r, or, TypeError "
+                    "for arguments %r: %s" % (self._name, args, err))
             # Somebody passed a non-string; I don't feel confident
             # guessing what they were trying to do.
             raise
@@ -104,12 +106,12 @@ class ParameterizingString(six.text_type):
             # ignore 'tparm() returned NULL', you won't get any styling,
             # even if does_styling is True. This happens on win32 platforms
             # with http://www.lfd.uci.edu/~gohlke/pythonlibs/#curses installed
-            if "tparm() returned NULL" not in six.text_type(err):
+            if "tparm() returned NULL" not in TextType(err):
                 raise
             return NullCallableString()
 
 
-class ParameterizingProxyString(six.text_type):
+class ParameterizingProxyString(TextType):
     r"""
     A Unicode string which can be called to proxy missing termcap entries.
 
@@ -133,22 +135,23 @@ class ParameterizingProxyString(six.text_type):
         u'\x1b[10G'
     """
 
-    def __new__(cls, *args):
+    def __new__(cls, fmt_pair, normal=u'', name=u'<not specified>'):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
         """
         Class constructor accepting 4 positional arguments.
 
-        :arg fmt: format string suitable for displaying terminal sequences.
-        :arg callable: receives __call__ arguments for formatting fmt.
-        :arg normal: terminating sequence for this capability (optional).
-        :arg name: name of this terminal capability (optional).
+        :arg tuple fmt_pair: Two element tuple containing:
+            - format string suitable for displaying terminal sequences
+            - callable suitable for receiving  __call__ arguments for formatting string
+        :arg str normal: terminating sequence for this capability (optional).
+        :arg str name: name of this terminal capability (optional).
         """
-        assert args and len(args) < 4, args
-        assert isinstance(args[0], tuple), args[0]
-        assert callable(args[0][1]), args[0][1]
-        new = six.text_type.__new__(cls, args[0][0])
-        new._fmt_args = args[0][1]
-        new._normal = args[1] if len(args) > 1 else u''
-        new._name = args[2] if len(args) > 2 else u'<not specified>'
+        assert isinstance(fmt_pair, tuple), fmt_pair
+        assert callable(fmt_pair[1]), fmt_pair[1]
+        new = TextType.__new__(cls, fmt_pair[0])
+        new._fmt_args = fmt_pair[1]
+        new._normal = normal
+        new._name = name
         return new
 
     def __call__(self, *args):
@@ -161,11 +164,13 @@ class ParameterizingProxyString(six.text_type):
         given capability.
 
         :rtype: FormattingString
+        :returns: Callable string for given parameters
         """
         return FormattingString(self.format(*self._fmt_args(*args)),
                                 self._normal)
 
-class FormattingString(six.text_type):
+
+class FormattingString(TextType):
     r"""
     A Unicode string which doubles as a callable.
 
@@ -175,6 +180,8 @@ class FormattingString(six.text_type):
     given (string) argument with the 2nd argument used by the class
     constructor::
 
+        >>> from blessed import Terminal
+        >>> term = Terminal()
         >>> style = FormattingString(term.bright_blue, term.normal)
         >>> print(repr(style))
         u'\x1b[94m'
@@ -182,34 +189,37 @@ class FormattingString(six.text_type):
         u'\x1b[94mBig Blue\x1b(B\x1b[m'
     """
 
-    def __new__(cls, *args):
+    def __new__(cls, sequence, normal=u''):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
         """
         Class constructor accepting 2 positional arguments.
 
-        :arg sequence: terminal attribute sequence.
-        :arg normal: terminating sequence for this attribute (optional).
+        :arg str sequence: terminal attribute sequence.
+        :arg str normal: terminating sequence for this attribute (optional).
         """
-        assert 1 <= len(args) <= 2, args
-        new = six.text_type.__new__(cls, args[0])
-        new._normal = args[1] if len(args) > 1 else u''
+        new = TextType.__new__(cls, sequence)
+        new._normal = normal
         return new
 
     def __call__(self, *args):
-        """Return ``text`` joined by ``sequence`` and ``normal``."""
+        """
+        Return ``text`` joined by ``sequence`` and ``normal``.
+
+        :raises TypeError: Not a string type
+        :rtype: str
+        :returns: Arguments wrapped in sequence and normal
+        """
         # Jim Allman brings us this convenience of allowing existing
         # unicode strings to be joined as a call parameter to a formatting
         # string result, allowing nestation:
         #
         # >>> t.red('This is ', t.bold('extremely'), ' dangerous!')
         for idx, ucs_part in enumerate(args):
-            if not isinstance(ucs_part, six.string_types):
-                raise TypeError("Positional argument #{idx} is {is_type} "
-                                "expected any of {expected_types}: "
-                                "{ucs_part!r}".format(
-                                    idx=idx, ucs_part=ucs_part,
-                                    is_type=type(ucs_part),
-                                    expected_types=six.string_types,
-                                ))
+            if not isinstance(ucs_part, StringType):
+                raise TypeError(
+                    "TypeError for FormattingString argument, %r, at position %s: expected type "
+                    "%s, got %s" % (ucs_part, idx, StringType.__name__, type(ucs_part).__name__)
+                )
         postfix = u''
         if self and self._normal:
             postfix = self._normal
@@ -220,12 +230,15 @@ class FormattingString(six.text_type):
         return self + u''.join(args) + postfix
 
 
-class FormattingOtherString(six.text_type):
+class FormattingOtherString(TextType):
     r"""
     A Unicode string which doubles as a callable for another sequence when called.
 
-    This is used for the :meth:`~.Terminal.move_up`, ``down``, ``left``, and ``right()`` family of functions::
+    This is used for the :meth:`~.Terminal.move_up`, ``down``, ``left``, and ``right()``
+    family of functions::
 
+        >>> from blessed import Terminal
+        >>> term = Terminal()
         >>> move_right = FormattingOtherString(term.cuf1, term.cuf)
         >>> print(repr(move_right))
         u'\x1b[C'
@@ -235,25 +248,28 @@ class FormattingOtherString(six.text_type):
         u'\x1b[C'
     """
 
-    def __new__(cls, *args):
+    def __new__(cls, direct, target):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
         """
         Class constructor accepting 2 positional arguments.
 
         :arg str direct: capability name for direct formatting, eg ``('x' + term.right)``.
-        :arg str callable: capability name for callable, eg ``('x' + term.right(99))``.
+        :arg str target: capability name for callable, eg ``('x' + term.right(99))``.
         """
-        assert 2 == len(args), args
-        new = six.text_type.__new__(cls, args[0])
-        new._callable = args[1]
+        new = TextType.__new__(cls, direct)
+        new._callable = target
         return new
 
-    def __call__(self, *args):
-        """Return ``text`` by ``callable``."""
-        if args:
-            return self._callable(*args)
-        return self
+    def __getnewargs__(self):
+        # return arguments used for the __new__ method upon unpickling.
+        return TextType.__new__(TextType, self), self._callable
 
-class NullCallableString(six.text_type):
+    def __call__(self, *args):
+        """Return ``text`` by ``target``."""
+        return self._callable(*args) if args else self
+
+
+class NullCallableString(TextType):
     """
     A dummy callable Unicode alternative to :class:`FormattingString`.
 
@@ -263,21 +279,19 @@ class NullCallableString(six.text_type):
 
     def __new__(cls):
         """Class constructor."""
-        new = six.text_type.__new__(cls, u'')
-        return new
+        return TextType.__new__(cls, u'')
 
     def __call__(self, *args):
         """
         Allow empty string to be callable, returning given string, if any.
 
-        When called with an int as the first arg, return an empty Unicode. An
-        int is a good hint that I am a :class:`ParameterizingString`, as there
-        are only about half a dozen string-returning capabilities listed in
-        terminfo(5) which accept non-int arguments, they are seldom used.
+        When called with an int as the first arg, return an empty Unicode. An int is a good hint
+        that I am a :class:`ParameterizingString`, as there are only about half a dozen string-
+        returning capabilities listed in terminfo(5) which accept non-int arguments, they are seldom
+        used.
 
-        When called with a non-int as the first arg (no no args at all), return
-        the first arg, acting in place of :class:`FormattingString` without
-        any attributes.
+        When called with a non-int as the first arg (no no args at all), return the first arg,
+        acting in place of :class:`FormattingString` without any attributes.
         """
         if not args or isinstance(args[0], int):
             # As a NullCallableString, even when provided with a parameter,
@@ -335,7 +349,7 @@ def get_proxy_string(term, attr):
 
 def split_compound(compound):
     """
-    Split compound formating string into segments.
+    Split compound formatting string into segments.
 
     >>> split_compound('bold_underline_bright_blue_on_red')
     ['bold', 'underline', 'bright_blue', 'on_red']
@@ -343,6 +357,7 @@ def split_compound(compound):
     :arg str compound: a string that may contain compounds, separated by
         underline (``_``).
     :rtype: list
+    :returns: List of formatting string segments
     """
     merged_segs = []
     # These occur only as prefixes, so they can always be merged:
@@ -366,11 +381,11 @@ def resolve_capability(term, attr):
        given :attr:`~.Terminal.kind`.
     :rtype: str
     """
-    # Decode sequences as latin1, as they are always 8-bit bytes, so when
-    # b'\xff' is returned, this must be decoded to u'\xff'.
     if not term.does_styling:
         return u''
-    val = curses.tigetstr(term._sugar.get(attr, attr))
+    val = curses.tigetstr(term._sugar.get(attr, attr))  # pylint: disable=protected-access
+    # Decode sequences as latin1, as they are always 8-bit bytes, so when
+    # b'\xff' is returned, this is decoded as u'\xff'.
     return u'' if val is None else val.decode('latin1')
 
 
@@ -390,6 +405,7 @@ def resolve_color(term, color):
         otherwise :class:`FormattingString`.
     :rtype: :class:`NullCallableString` or :class:`FormattingString`
     """
+    # pylint: disable=protected-access
     if term.number_of_colors == 0:
         return NullCallableString()
 
@@ -433,7 +449,7 @@ def resolve_attribute(term, attr):
     :arg Terminal term: :class:`~.Terminal` instance.
     :arg str attr: Sugary, ordinary, or compound formatted terminal
         capability, such as "red_on_white", "normal", "red", or
-        "bold_on_black", respectively.
+        "bold_on_black".
     :returns: a string class instance which emits the terminal sequence
         for the given terminal capability, or may be used as a callable to
         wrap the given string with such sequence.
@@ -454,7 +470,7 @@ def resolve_attribute(term, attr):
     # call for each compounding section, joined and returned as
     # a completed completed FormattingString.
     formatters = split_compound(attr)
-    if all(fmt in (COLORS | COMPOUNDABLES) for fmt in formatters):
+    if all((fmt in COLORS or fmt in COMPOUNDABLES) for fmt in formatters):
         resolution = (resolve_attribute(term, fmt) for fmt in formatters)
         return FormattingString(u''.join(resolution), term.normal)
 
@@ -467,7 +483,8 @@ def resolve_attribute(term, attr):
         # and, for special terminals, such as 'screen', provide a Proxy
         # ParameterizingString for attributes they do not claim to support,
         # but actually do! (such as 'hpa' and 'vpa').
-        proxy = get_proxy_string(term, term._sugar.get(attr, attr))
+        proxy = get_proxy_string(term,
+                                 term._sugar.get(attr, attr))  # pylint: disable=protected-access
         if proxy is not None:
             return proxy
 
