@@ -1,25 +1,26 @@
-# encoding: utf-8
+# -*- coding: utf-8 -*-
 """Module providing 'sequence awareness'."""
 # std imports
 import re
 import math
 import textwrap
-import functools
 
 # 3rd party
-import six
-import wcwidth
+from wcwidth import wcwidth
 
 # local
-from blessed._capabilities import CAPABILITIES_CAUSE_MOVEMENT
+from blessed._compat import TextType
+from blessed._capabilities import CAPABILITIES_CAUSE_MOVEMENT, CAPABILITIES_HORIZONTAL_DISTANCE
 
 __all__ = ('Sequence', 'SequenceTextWrapper', 'iter_parse', 'measure_length')
 
 
+# pylint: disable=unused-argument,missing-function-docstring
+
 class Termcap(object):
     """Terminal capability of given variable name and pattern."""
 
-    def __init__(self, name, pattern, attribute):
+    def __init__(self, name, pattern, attribute, nparams=0):
         """
         Class initializer.
 
@@ -27,10 +28,12 @@ class Termcap(object):
         :arg str pattern: regular expression string.
         :arg str attribute: :class:`~.Terminal` attribute used to build
             this terminal capability.
+        :arg int nparams: number of positional arguments for callable.
         """
         self.name = name
         self.pattern = pattern
         self.attribute = attribute
+        self.nparams = nparams
         self._re_compiled = None
 
     def __repr__(self):
@@ -39,11 +42,13 @@ class Termcap(object):
 
     @property
     def named_pattern(self):
+        """Regular expression pattern for capability with named group."""
         # pylint: disable=redundant-keyword-arg
         return '(?P<{self.name}>{self.pattern})'.format(self=self)
 
     @property
     def re_compiled(self):
+        """Compiled regular expression pattern for capability."""
         if self._re_compiled is None:
             self._re_compiled = re.compile(self.pattern)
         return self._re_compiled
@@ -58,33 +63,20 @@ class Termcap(object):
         Horizontal carriage adjusted by capability, may be negative.
 
         :rtype: int
-        :arg str text: for capabilities *parm_left_cursor*,
-            *parm_right_cursor*, provide the matching sequence
-            text, its interpreted distance is returned.
-
+        :arg str text: for capabilities *parm_left_cursor*, *parm_right_cursor*, provide the
+            matching sequence text, its interpreted distance is returned.
         :returns: 0 except for matching '
         """
-        value = {
-            'cursor_left': -1,
-            'backspace': -1,
-            'cursor_right': 1,
-            'tab': 8,
-            'ascii_tab': 8,
-        }.get(self.name, None)
-        if value is not None:
-            return value
+        value = CAPABILITIES_HORIZONTAL_DISTANCE.get(self.name)
+        if value is None:
+            return 0
 
-        unit = {
-            'parm_left_cursor': -1,
-            'parm_right_cursor': 1
-        }.get(self.name, None)
-        if unit is not None:
-            value = int(self.re_compiled.match(text).group(1))
-            return unit * value
+        if self.nparams:
+            return value * int(self.re_compiled.match(text).group(1))
 
-        return 0
+        return value
 
-    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-positional-arguments
     @classmethod
     def build(cls, name, capability, attribute, nparams=0,
               numeric=99, match_grouped=False, match_any=False,
@@ -97,9 +89,10 @@ class Termcap(object):
             capability to build for. When ``nparams`` is non-zero, it
             must be a callable unicode string (such as the result from
             ``getattr(term, 'bold')``.
-        :arg attribute: The terminfo(5) capability name by which this
+        :arg str attribute: The terminfo(5) capability name by which this
             pattern is known.
         :arg int nparams: number of positional arguments for callable.
+        :arg int numeric: Value to substitute into capability to when generating pattern
         :arg bool match_grouped: If the numeric pattern should be
             grouped, ``(\d+)`` when ``True``, ``\d+`` default.
         :arg bool match_any: When keyword argument ``nparams`` is given,
@@ -109,6 +102,8 @@ class Termcap(object):
             pattern ``(\d+)`` in builder.
         :arg bool match_optional: When ``True``, building of numeric patterns
             containing ``(\d+)`` will be built as optional, ``(\d+)?``.
+        :rtype: blessed.sequences.Termcap
+        :returns: Terminal capability instance for given capability definition
         """
         _numeric_regex = r'\d+'
         if match_grouped:
@@ -119,7 +114,7 @@ class Termcap(object):
 
         # basic capability attribute, not used as a callable
         if nparams == 0:
-            return cls(name, re.escape(capability), attribute)
+            return cls(name, re.escape(capability), attribute, nparams)
 
         # a callable capability accepting numeric argument
         _outp = re.escape(capability(*(numeric,) * nparams))
@@ -127,13 +122,13 @@ class Termcap(object):
             for num in range(numeric - 1, numeric + 2):
                 if str(num) in _outp:
                     pattern = _outp.replace(str(num), _numeric_regex)
-                    return cls(name, pattern, attribute)
+                    return cls(name, pattern, attribute, nparams)
 
         if match_grouped:
             pattern = re.sub(r'(\d+)', lambda x: _numeric_regex, _outp)
         else:
             pattern = re.sub(r'\d+', lambda x: _numeric_regex, _outp)
-        return cls(name, pattern, attribute)
+        return cls(name, pattern, attribute, nparams)
 
 
 class SequenceTextWrapper(textwrap.TextWrapper):
@@ -152,6 +147,10 @@ class SequenceTextWrapper(textwrap.TextWrapper):
         """
         Sequence-aware variant of :meth:`textwrap.TextWrapper._wrap_chunks`.
 
+        :raises ValueError: ``self.width`` is not a positive integer
+        :rtype: list
+        :returns: text chunks adjusted for width
+
         This simply ensures that word boundaries are not broken mid-sequence, as standard python
         textwrap would incorrectly determine the length of a string containing sequences, and may
         also break consider sequences part of a "word" that may be broken by hyphen (``-``), where
@@ -170,23 +169,19 @@ class SequenceTextWrapper(textwrap.TextWrapper):
         while chunks:
             cur_line = []
             cur_len = 0
-            if lines:
-                indent = self.subsequent_indent
-            else:
-                indent = self.initial_indent
+            indent = self.subsequent_indent if lines else self.initial_indent
             width = self.width - len(indent)
             if drop_whitespace and (
                     Sequence(chunks[-1], term).strip() == '' and lines):
                 del chunks[-1]
             while chunks:
                 chunk_len = Sequence(chunks[-1], term).length()
-                if cur_len + chunk_len <= width:
-                    cur_line.append(chunks.pop())
-                    cur_len += chunk_len
-                else:
+                if cur_len + chunk_len > width:
+                    if chunk_len > width:
+                        self._handle_long_word(chunks, cur_line, cur_len, width)
                     break
-            if chunks and Sequence(chunks[-1], term).length() > width:
-                self._handle_long_word(chunks, cur_line, cur_len, width)
+                cur_line.append(chunks.pop())
+                cur_len += chunk_len
             if drop_whitespace and (
                     cur_line and Sequence(cur_line[-1], term).strip() == ''):
                 del cur_line[-1]
@@ -198,29 +193,40 @@ class SequenceTextWrapper(textwrap.TextWrapper):
         """
         Sequence-aware :meth:`textwrap.TextWrapper._handle_long_word`.
 
-        This simply ensures that word boundaries are not broken mid-sequence, as standard python
-        textwrap would incorrectly determine the length of a string containing sequences, and may
-        also break consider sequences part of a "word" that may be broken by hyphen (``-``), where
-        this implementation corrects both.
+        This method ensures that word boundaries are not broken mid-sequence, as
+        standard python textwrap would incorrectly determine the length of a
+        string containing sequences and wide characters it would also break
+        these "words" that would be broken by hyphen (``-``), this
+        implementation corrects both.
+
+        This is done by mutating the passed arguments, removing items from
+        'reversed_chunks' and appending them to 'cur_line'.
+
+        However, some characters (east-asian, emoji, etc.) cannot be split any
+        less than 2 cells, so in the case of a width of 1, we have no choice
+        but to allow those characters to flow outside of the given cell.
         """
         # Figure out when indent is larger than the specified width, and make
         # sure at least one character is stripped off on every pass
-        if width < 1:
-            space_left = 1
-        else:
-            space_left = width - cur_len
-
+        space_left = 1 if width < 1 else width - cur_len
         # If we're allowed to break long words, then do so: put as much
         # of the next chunk onto the current line as will fit.
 
         if self.break_long_words:
             term = self.term
             chunk = reversed_chunks[-1]
-            idx = nxt = 0
+            idx = nxt = seq_length = 0
             for text, _ in iter_parse(term, chunk):
                 nxt += len(text)
-                if Sequence(chunk[:nxt], term).length() > space_left:
-                    break
+                seq_length += Sequence(text, term).length()
+                if seq_length > space_left:
+                    if cur_len == 0 and width == 1 and nxt == 1 and seq_length == 2:
+                        # Emoji etc. cannot be split under 2 cells, so in the
+                        # case of a width of 1, we have no choice but to allow
+                        # those characters to flow outside of the given cell.
+                        pass
+                    else:
+                        break
                 idx = nxt
             cur_line.append(chunk[:idx])
             reversed_chunks[-1] = chunk[idx:]
@@ -241,7 +247,7 @@ class SequenceTextWrapper(textwrap.TextWrapper):
 SequenceTextWrapper.__doc__ = textwrap.TextWrapper.__doc__
 
 
-class Sequence(six.text_type):
+class Sequence(TextType):
     """
     A "sequence-aware" version of the base :class:`str` class.
 
@@ -251,13 +257,14 @@ class Sequence(six.text_type):
     """
 
     def __new__(cls, sequence_text, term):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
         """
         Class constructor.
 
-        :arg sequence_text: A string that may contain sequences.
+        :arg str sequence_text: A string that may contain sequences.
         :arg blessed.Terminal term: :class:`~.Terminal` instance.
         """
-        new = six.text_type.__new__(cls, sequence_text)
+        new = TextType.__new__(cls, sequence_text)
         new._term = term
         return new
 
@@ -265,14 +272,14 @@ class Sequence(six.text_type):
         """
         Return string containing sequences, left-adjusted.
 
-        :arg int width: Total width given to right-adjust ``text``.  If
+        :arg int width: Total width given to left-adjust ``text``.  If
             unspecified, the width of the attached terminal is used (default).
         :arg str fillchar: String for padding right-of ``text``.
-        :returns: String of ``text``, right-aligned by ``width``.
+        :returns: String of ``text``, left-aligned by ``width``.
         :rtype: str
         """
         rightside = fillchar * int(
-            (max(0.0, float(width - self.length()))) / float(len(fillchar)))
+            (max(0.0, float(width.__index__() - self.length()))) / float(len(fillchar)))
         return u''.join((self, rightside))
 
     def rjust(self, width, fillchar=u' '):
@@ -286,7 +293,7 @@ class Sequence(six.text_type):
         :rtype: str
         """
         leftside = fillchar * int(
-            (max(0.0, float(width - self.length()))) / float(len(fillchar)))
+            (max(0.0, float(width.__index__() - self.length()))) / float(len(fillchar)))
         return u''.join((leftside, self))
 
     def center(self, width, fillchar=u' '):
@@ -299,13 +306,42 @@ class Sequence(six.text_type):
         :returns: String of ``text``, centered by ``width``.
         :rtype: str
         """
-        split = max(0.0, float(width) - self.length()) / 2
+        split = max(0.0, float(width.__index__()) - self.length()) / 2
         leftside = fillchar * int(
             (max(0.0, math.floor(split))) / float(len(fillchar)))
         rightside = fillchar * int(
             (max(0.0, math.ceil(split))) / float(len(fillchar)))
         return u''.join((leftside, self, rightside))
 
+    def truncate(self, width):
+        """
+        Truncate a string in a sequence-aware manner.
+
+        Any printable characters beyond ``width`` are removed, while all
+        sequences remain in place. Horizontal Sequences are first expanded
+        by :meth:`padd`.
+
+        :arg int width: The printable width to truncate the string to.
+        :rtype: str
+        :returns: String truncated to at most ``width`` printable characters.
+        """
+        output = ""
+        current_width = 0
+        target_width = width.__index__()
+        parsed_seq = iter_parse(self._term, self.padd())
+
+        # Retain all text until non-cap width reaches desired width
+        for text, cap in parsed_seq:
+            if not cap:
+                # use wcwidth clipped to 0 because it can sometimes return -1
+                current_width += max(wcwidth(text), 0)
+                if current_width > target_width:
+                    break
+            output += text
+
+        # Return with remaining caps appended
+        return output + ''.join(text for text, cap in parsed_seq if cap)
+
     def length(self):
         r"""
         Return the printable length of string containing sequences.
@@ -318,37 +354,30 @@ class Sequence(six.text_type):
         Some characters may consume more than one cell, mainly those CJK
         Unified Ideographs (Chinese, Japanese, Korean) defined by Unicode
         as half or full-width characters.
+
+        For example:
+
+            >>> from blessed import Terminal
+            >>> from blessed.sequences import Sequence
+            >>> term = Terminal()
+            >>> msg = term.clear + term.red(u'コンニチハ')
+            >>> Sequence(msg, term).length()
+            10
+
+        .. note:: Although accounted for, strings containing sequences such
+            as ``term.clear`` will not give accurate returns, it is not
+            considered lengthy (a length of 0).
         """
         # because control characters may return -1, "clip" their length to 0.
-        clip = functools.partial(max, 0)
-        return sum(clip(wcwidth.wcwidth(w_char))
-                   for w_char in self.strip_seqs())
-
-    # we require ur"" for the docstring, but it is not supported by all
-    # python versions.
-    if length.__doc__ is not None:
-        length.__doc__ += (
-            u"""
-            For example:
-
-                >>> from blessed import Terminal
-                >>> from blessed.sequences import Sequence
-                >>> term = Terminal()
-                >>> msg = term.clear + term.red(u'コンニチハ'), term
-                >>> Sequence(msg).length()
-                10
-
-            .. note:: Although accounted for, strings containing sequences such
-               as ``term.clear`` will not give accurate returns, it is not
-               considered lengthy (a length of 0).
-            """)
+        return sum(max(wcwidth(w_char), 0) for w_char in self.padd(strip=True))
 
     def strip(self, chars=None):
         """
-        Return string of sequences, leading, and trailing whitespace removed.
+        Return string of sequences, leading and trailing whitespace removed.
 
         :arg str chars: Remove characters in chars instead of whitespace.
         :rtype: str
+        :returns: string of sequences with leading and trailing whitespace removed.
         """
         return self.strip_seqs().strip(chars)
 
@@ -358,6 +387,7 @@ class Sequence(six.text_type):
 
         :arg str chars: Remove characters in chars instead of whitespace.
         :rtype: str
+        :returns: string of sequences with leading removed.
         """
         return self.strip_seqs().lstrip(chars)
 
@@ -367,6 +397,7 @@ class Sequence(six.text_type):
 
         :arg str chars: Remove characters in chars instead of whitespace.
         :rtype: str
+        :returns: string of sequences with trailing removed.
         """
         return self.strip_seqs().rstrip(chars)
 
@@ -375,29 +406,54 @@ class Sequence(six.text_type):
         Return ``text`` stripped of only its terminal sequences.
 
         :rtype: str
+        :returns: Text with terminal sequences removed
         """
-        gen = iter_parse(self._term, self.padd())
-        return u''.join(text for text, cap in gen if not cap)
+        return self.padd(strip=True)
 
-    def padd(self):
+    def padd(self, strip=False):
         """
         Return non-destructive horizontal movement as destructive spacing.
 
+        :arg bool strip: Strip terminal sequences
         :rtype: str
+        :returns: Text adjusted for horizontal movement
         """
+        data = self
+        if self._term.caps_compiled.search(data) is None:
+            return TextType(data)
+        if strip:  # strip all except CAPABILITIES_HORIZONTAL_DISTANCE
+            # pylint: disable-next=protected-access
+            data = self._term._caps_compiled_without_hdist.sub("", data)
+
+            if self._term.caps_compiled.search(data) is None:
+                return TextType(data)
+
         outp = ''
-        for text, cap in iter_parse(self._term, self):
-            if not cap:
-                outp += text
-                continue
+        last_end = 0
+
+        # pylint: disable-next=protected-access
+        for match in self._term._caps_named_compiled.finditer(data):
+
+            # Capture unmatched text between matched capabilities
+            if match.start() > last_end:
+                outp += data[last_end:match.start()]
+
+            last_end = match.end()
+
+            text = match.group(match.lastgroup)
+            value = self._term.caps[match.lastgroup].horizontal_distance(text)
 
-            value = cap.horizontal_distance(text)
             if value > 0:
                 outp += ' ' * value
             elif value < 0:
                 outp = outp[:value]
             else:
                 outp += text
+
+        # Capture any remaining unmatched text
+        if last_end < len(data):
+            outp += data[last_end:]
+
         return outp
 
 
@@ -409,7 +465,7 @@ def iter_parse(term, text):
     :class:`str` of length 1.  Otherwise, ``text`` is a full
     matching sequence of given capability.
     """
-    for match in re.finditer(term._caps_compiled_any, text):
+    for match in term._caps_compiled_any.finditer(text):  # pylint: disable=protected-access
         name = match.lastgroup
         value = match.group(name)
         if name == 'MISMATCH':
@@ -423,6 +479,7 @@ def measure_length(text, term):
     .. deprecated:: 1.12.0.
 
     :rtype: int
+    :returns: Length of the first sequence in the string
     """
     try:
         text, capability = next(iter_parse(term, text))
