@@ -12,13 +12,13 @@ import select
 import struct
 import platform
 import warnings
-import functools
 import contextlib
 import collections
 
 # local
 from .color import COLOR_DISTANCE_ALGORITHMS
-from .keyboard import (_time_left,
+from .keyboard import (DEFAULT_ESCDELAY,
+                       _time_left,
                        _read_until,
                        resolve_sequence,
                        get_keyboard_codes,
@@ -35,25 +35,30 @@ from .formatters import (COLORS,
                          split_compound,
                          resolve_attribute,
                          resolve_capability)
-from ._capabilities import CAPABILITY_DATABASE, CAPABILITIES_ADDITIVES, CAPABILITIES_RAW_MIXIN
+from ._capabilities import (CAPABILITY_DATABASE,
+                            CAPABILITIES_ADDITIVES,
+                            CAPABILITIES_RAW_MIXIN,
+                            CAPABILITIES_HORIZONTAL_DISTANCE)
 
-try:
-    InterruptedError
-except NameError:
-    # alias py2 exception to py3
-    # pylint: disable=redefined-builtin
-    InterruptedError = select.error
+# isort: off
+
+# Alias py2 exception to py3
+if sys.version_info[:2] < (3, 3):
+    InterruptedError = select.error  # pylint: disable=redefined-builtin
 
 
 HAS_TTY = True
 if platform.system() == 'Windows':
+    IS_WINDOWS = True
     import jinxed as curses  # pylint: disable=import-error
+    from jinxed.win32 import get_console_input_encoding  # pylint: disable=import-error
 else:
+    IS_WINDOWS = False
     import curses
 
     try:
-        import termios
         import fcntl
+        import termios
         import tty
     except ImportError:
         _TTY_METHODS = ('setraw', 'cbreak', 'kbhit', 'height', 'width')
@@ -84,40 +89,40 @@ class Terminal(object):
     #         Too many instance attributes (12/7)
 
     #: Sugary names for commonly-used capabilities
-    _sugar = dict(
-        save='sc',
-        restore='rc',
-        clear_eol='el',
-        clear_bol='el1',
-        clear_eos='ed',
-        enter_fullscreen='smcup',
-        exit_fullscreen='rmcup',
-        move='cup',
-        move_yx='cup',
-        move_x='hpa',
-        move_y='vpa',
-        hide_cursor='civis',
-        normal_cursor='cnorm',
-        reset_colors='op',
-        normal='sgr0',
-        reverse='rev',
-        italic='sitm',
-        no_italic='ritm',
-        shadow='sshm',
-        no_shadow='rshm',
-        standout='smso',
-        no_standout='rmso',
-        subscript='ssubm',
-        no_subscript='rsubm',
-        superscript='ssupm',
-        no_superscript='rsupm',
-        underline='smul',
-        no_underline='rmul',
-        cursor_report='u6',
-        cursor_request='u7',
-        terminal_answerback='u8',
-        terminal_enquire='u9',
-    )
+    _sugar = {
+        'save': 'sc',
+        'restore': 'rc',
+        'clear_eol': 'el',
+        'clear_bol': 'el1',
+        'clear_eos': 'ed',
+        'enter_fullscreen': 'smcup',
+        'exit_fullscreen': 'rmcup',
+        'move': 'cup',
+        'move_yx': 'cup',
+        'move_x': 'hpa',
+        'move_y': 'vpa',
+        'hide_cursor': 'civis',
+        'normal_cursor': 'cnorm',
+        'reset_colors': 'op',
+        'normal': 'sgr0',
+        'reverse': 'rev',
+        'italic': 'sitm',
+        'no_italic': 'ritm',
+        'shadow': 'sshm',
+        'no_shadow': 'rshm',
+        'standout': 'smso',
+        'no_standout': 'rmso',
+        'subscript': 'ssubm',
+        'no_subscript': 'rsubm',
+        'superscript': 'ssupm',
+        'no_superscript': 'rsupm',
+        'underline': 'smul',
+        'no_underline': 'rmul',
+        'cursor_report': 'u6',
+        'cursor_request': 'u7',
+        'terminal_answerback': 'u8',
+        'terminal_enquire': 'u9',
+    }
 
     def __init__(self, kind=None, stream=None, force_styling=False):
         """
@@ -126,7 +131,7 @@ class Terminal(object):
         :arg str kind: A terminal string as taken by :func:`curses.setupterm`.
             Defaults to the value of the ``TERM`` environment variable.
 
-            .. note:: Terminals withing a single process must share a common
+            .. note:: Terminals within a single process must share a common
                 ``kind``. See :obj:`_CUR_TERM`.
 
         :arg file stream: A file-like object representing the Terminal output.
@@ -161,19 +166,16 @@ class Terminal(object):
         self._is_a_tty = False
         self.__init__streams()
 
-        if platform.system() == 'Windows' and self._init_descriptor is not None:
+        if IS_WINDOWS and self._init_descriptor is not None:
             self._kind = kind or curses.get_term(self._init_descriptor)
         else:
             self._kind = kind or os.environ.get('TERM', 'dumb') or 'dumb'
 
         self._does_styling = False
-        if force_styling:
+        if force_styling is None and self.is_a_tty:
+            self.errors.append('force_styling is None')
+        elif force_styling or self.is_a_tty:
             self._does_styling = True
-        elif self.is_a_tty:
-            if force_styling is None:
-                self.errors.append('force_styling is None')
-            else:
-                self._does_styling = True
 
         if self.does_styling:
             # Initialize curses (call setupterm), so things like tigetstr() work.
@@ -259,8 +261,7 @@ class Terminal(object):
         self._color_distance_algorithm = 'cie2000'
         if not self.does_styling:
             self.number_of_colors = 0
-        elif platform.system() == 'Windows' or (
-                os.environ.get('COLORTERM') in ('truecolor', '24bit')):
+        elif IS_WINDOWS or os.environ.get('COLORTERM') in ('truecolor', '24bit'):
             self.number_of_colors = 1 << 24
         else:
             self.number_of_colors = max(0, curses.tigetnum('colors') or -1)
@@ -275,8 +276,8 @@ class Terminal(object):
         self.caps = collections.OrderedDict()
 
         # some static injected patterns, esp. without named attribute access.
-        for name, (attribute, pattern) in CAPABILITIES_ADDITIVES.items():
-            self.caps[name] = Termcap(name, pattern, attribute)
+        for name, args in CAPABILITIES_ADDITIVES.items():
+            self.caps[name] = Termcap(name, *args)
 
         for name, (attribute, kwds) in CAPABILITY_DATABASE.items():
             if self.does_styling:
@@ -290,19 +291,28 @@ class Terminal(object):
             # fall-back
             pattern = CAPABILITIES_RAW_MIXIN.get(name)
             if pattern:
-                self.caps[name] = Termcap(name, pattern, attribute)
+                self.caps[name] = Termcap(name, pattern, attribute, kwds.get('nparams', 0))
 
         # make a compiled named regular expression table
         self.caps_compiled = re.compile(
-            '|'.join(cap.pattern for name, cap in self.caps.items()))
-
+            '|'.join(cap.pattern for cap in self.caps.values())
+        )
+        # Used with padd() to separate plain text from caps
+        self._caps_named_compiled = re.compile(
+            '|'.join(cap.named_pattern for cap in self.caps.values())
+        )
+        # Used with padd() to strip non-horizontal caps
+        self._caps_compiled_without_hdist = re.compile('|'.join(
+            cap.pattern for cap in self.caps.values()
+            if cap.name not in CAPABILITIES_HORIZONTAL_DISTANCE)
+        )
         # for tokenizer, the '.lastgroup' is the primary lookup key for
         # 'self.caps', unless 'MISMATCH'; then it is an unmatched character.
         self._caps_compiled_any = re.compile('|'.join(
-            cap.named_pattern for name, cap in self.caps.items()
+            cap.named_pattern for cap in self.caps.values()
         ) + '|(?P<MISMATCH>.)')
         self._caps_unnamed_any = re.compile('|'.join(
-            '({0})'.format(cap.pattern) for name, cap in self.caps.items()
+            '({0})'.format(cap.pattern) for cap in self.caps.values()
         ) + '|(.)')
 
     def __init__keycodes(self):
@@ -325,7 +335,14 @@ class Terminal(object):
 
         if self._keyboard_fd is not None:
             # set input encoding and initialize incremental decoder
-            self._encoding = locale.getpreferredencoding() or 'UTF-8'
+
+            if IS_WINDOWS:
+                # pylint: disable-next=possibly-used-before-assignment
+                self._encoding = get_console_input_encoding() \
+                    or locale.getpreferredencoding() or 'UTF-8'
+            else:
+                self._encoding = locale.getpreferredencoding() or 'UTF-8'
+
             try:
                 self._keyboard_decoder = codecs.getincrementaldecoder(self._encoding)()
             except LookupError as err:
@@ -416,7 +433,7 @@ class Terminal(object):
     @property
     def pixel_height(self):
         """
-        Read-only property: Height ofthe terminal (in pixels).
+        Read-only property: Height of the terminal (in pixels).
 
         :rtype: int
         """
@@ -455,7 +472,7 @@ class Terminal(object):
             - ``ws_ypixel``: height of terminal by pixels (not accurate).
         """
         if HAS_TTY:
-            # pylint: disable=protected-access
+            # pylint: disable=protected-access,possibly-used-before-assignment
             data = fcntl.ioctl(fd, termios.TIOCGWINSZ, WINSZ._BUF)
             return WINSZ(*struct.unpack(WINSZ._FMT, data))
         return WINSZ(ws_row=25, ws_col=80, ws_xpixel=0, ws_ypixel=0)
@@ -497,6 +514,50 @@ class Terminal(object):
                      ws_xpixel=None,
                      ws_ypixel=None)
 
+    def _query_response(self, query_str, response_re, timeout):
+        """
+        Sends a query string to the terminal and waits for a response.
+
+        :arg str query_str: Query string written to output
+        :arg str response_re: Regular expression matching query response
+        :arg float timeout: Return after time elapsed in seconds
+        :return: re.match object for response_re or None if not found
+        :rtype: re.Match
+        """
+        # Avoid changing user's desired raw or cbreak mode if already entered,
+        # by entering cbreak mode ourselves.  This is necessary to receive user
+        # input without awaiting a human to press the return key.   This mode
+        # also disables echo, which we should also hide, as our input is an
+        # sequence that is not meaningful for display as an output sequence.
+
+        ctx = None
+        try:
+            if self._line_buffered:
+                ctx = self.cbreak()
+                ctx.__enter__()  # pylint: disable=no-member
+
+            # Emit the query sequence,
+            self.stream.write(query_str)
+            self.stream.flush()
+
+            # Wait for response
+            match, data = _read_until(term=self,  # pylint: disable=unpacking-non-sequence
+                                      pattern=response_re,
+                                      timeout=timeout)
+
+            # Exclude response from subsequent input
+            if match:
+                data = data[:match.start()] + data[match.end():]
+
+            # re-buffer keyboard data, if any
+            self.ungetch(data)
+
+        finally:
+            if ctx is not None:
+                ctx.__exit__(None, None, None)  # pylint: disable=no-member
+
+        return match
+
     @contextlib.contextmanager
     def location(self, x=None, y=None):
         """
@@ -529,7 +590,7 @@ class Terminal(object):
 
         .. note:: The argument order *(x, y)* differs from the return value order *(y, x)*
             of :meth:`get_location`, or argument order *(y, x)* of :meth:`move`. This is
-            for API Compaibility with the blessings library, sorry for the trouble!
+            for API Compatibility with the blessings library, sorry for the trouble!
         """
         # pylint: disable=invalid-name
         #         Invalid argument name "x"
@@ -561,11 +622,12 @@ class Terminal(object):
             always ensure the return value is checked for ``(-1, -1)``.
 
         The location of the cursor is determined by emitting the ``u7`` terminal capability, or
-        VT100 `Query Cursor Position <http://www.termsys.demon.co.uk/vtansi.htm#status>`_ when such
-        capability is undefined, which elicits a response from a reply string described by
+        VT100 `Query Cursor Position
+        <https://www2.ccs.neu.edu/research/gpc/VonaUtils/vona/terminal/vtansi.htm#status>`_
+        when such capability is undefined, which elicits a response from a reply string described by
         capability ``u6``, or again VT100's definition of ``\x1b[%i%d;%dR`` when undefined.
 
-        The ``(y, x)`` return value matches the parameter order of the :meth:`move_xy` capability.
+        The ``(y, x)`` return value matches the parameter order of the :meth:`move_yx` capability.
         The following sequence should cause the cursor to not move at all::
 
             >>> term = Terminal()
@@ -580,7 +642,6 @@ class Terminal(object):
             ...
             >>> assert given_x == result_x, (given_x, result_x)
             >>> assert given_y == result_y, (given_y, result_y)
-
         """
         # Local lines attached by termios and remote login protocols such as
         # ssh and telnet both provide a means to determine the window
@@ -598,65 +659,75 @@ class Terminal(object):
         # >  u8   terminal answerback description
         # >  u7   cursor position request (equiv. to VT100/ANSI/ECMA-48 DSR 6)
         # >  u6   cursor position report (equiv. to ANSI/ECMA-48 CPR)
-        query_str = self.u7 or u'\x1b[6n'
+
         response_str = getattr(self, self.caps['cursor_report'].attribute) or u'\x1b[%i%d;%dR'
+        match = self._query_response(
+            self.u7 or u'\x1b[6n', self.caps['cursor_report'].re_compiled, timeout
+        )
 
-        # determine response format as a regular expression
-        response_re = self.caps['cursor_report'].re_compiled
+        if match:
+            # return matching sequence response, the cursor location.
+            row, col = (int(val) for val in match.groups())
 
-        # Avoid changing user's desired raw or cbreak mode if already entered,
-        # by entering cbreak mode ourselves.  This is necessary to receive user
-        # input without awaiting a human to press the return key.   This mode
-        # also disables echo, which we should also hide, as our input is an
-        # sequence that is not meaningful for display as an output sequence.
-
-        ctx = None
-        try:
-            if self._line_buffered:
-                ctx = self.cbreak()
-                ctx.__enter__()  # pylint: disable=no-member
-
-            # emit the 'query cursor position' sequence,
-            self.stream.write(query_str)
-            self.stream.flush()
-
-            # expect a response,
-            match, data = _read_until(term=self,
-                                      pattern=response_re,
-                                      timeout=timeout)
-
-            # ensure response sequence is excluded from subsequent input,
-            if match:
-                data = (data[:match.start()] + data[match.end():])
-
-            # re-buffer keyboard data, if any
-            self.ungetch(data)
-
-            if match:
-                # return matching sequence response, the cursor location.
-                row, col = (int(val) for val in match.groups())
-
-                # Per https://invisible-island.net/ncurses/terminfo.src.html
-                # The cursor position report (<u6>) string must contain two
-                # scanf(3)-style %d format elements.  The first of these must
-                # correspond to the Y coordinate and the second to the %d.
-                # If the string contains the sequence %i, it is taken as an
-                # instruction to decrement each value after reading it (this is
-                # the inverse sense from the cup string).
-                if u'%i' in response_str:
-                    row -= 1
-                    col -= 1
-                return row, col
-
-        finally:
-            if ctx is not None:
-                ctx.__exit__(None, None, None)  # pylint: disable=no-member
+            # Per https://invisible-island.net/ncurses/terminfo.src.html
+            # The cursor position report (<u6>) string must contain two
+            # scanf(3)-style %d format elements.  The first of these must
+            # correspond to the Y coordinate and the second to the %d.
+            # If the string contains the sequence %i, it is taken as an
+            # instruction to decrement each value after reading it (this is
+            # the inverse sense from the cup string).
+            if u'%i' in response_str:
+                row -= 1
+                col -= 1
+            return row, col
 
         # We chose to return an illegal value rather than an exception,
         # favoring that users author function filters, such as max(0, y),
         # rather than crowbarring such logic into an exception handler.
         return -1, -1
 
+    def get_fgcolor(self, timeout=None):
+        """
+        Return tuple (r, g, b) of foreground color.
+
+        :arg float timeout: Return after time elapsed in seconds with value ``(-1, -1, -1)``
+            indicating that the remote end did not respond.
+        :rtype: tuple
+        :returns: foreground color as tuple in form of ``(r, g, b)``.  When a timeout is specified,
+            always ensure the return value is checked for ``(-1, -1, -1)``.
+
+        The foreground color is determined by emitting an `OSC 10 color query
+        <https://invisible-island.net/xterm/ctlseqs/ctlseqs.html#h3-Operating-System-Commands>`_.
+        """
+        match = self._query_response(
+            u'\x1b]10;?\x07',
+            re.compile(u'\x1b]10;rgb:([0-9a-fA-F]+)/([0-9a-fA-F]+)/([0-9a-fA-F]+)\x07'),
+            timeout
+        )
+
+        return tuple(int(val, 16) for val in match.groups()) if match else (-1, -1, -1)
+
+    def get_bgcolor(self, timeout=None):
+        """
+        Return tuple (r, g, b) of background color.
+
+        :arg float timeout: Return after time elapsed in seconds with value ``(-1, -1, -1)``
+            indicating that the remote end did not respond.
+        :rtype: tuple
+        :returns: background color as tuple in form of ``(r, g, b)``.  When a timeout is specified,
+            always ensure the return value is checked for ``(-1, -1, -1)``.
+
+        The background color is determined by emitting an `OSC 11 color query
+        <https://invisible-island.net/xterm/ctlseqs/ctlseqs.html#h3-Operating-System-Commands>`_.
+        """
+        match = self._query_response(
+            u'\x1b]11;?\x07',
+            re.compile(u'\x1b]11;rgb:([0-9a-fA-F]+)/([0-9a-fA-F]+)/([0-9a-fA-F]+)\x07'),
+            timeout
+        )
+
+        return tuple(int(val, 16) for val in match.groups()) if match else (-1, -1, -1)
+
     @contextlib.contextmanager
     def fullscreen(self):
         """
@@ -761,10 +832,10 @@ class Terminal(object):
         This should not be used directly, but rather a specific color by name or
         :meth:`~.Terminal.color_rgb` value.
         """
-        if not self.does_styling:
-            return NullCallableString()
-        return ParameterizingString(self._foreground_color,
-                                    self.normal, 'color')
+        if self.does_styling:
+            return ParameterizingString(self._foreground_color, self.normal, 'color')
+
+        return NullCallableString()
 
     def color_rgb(self, red, green, blue):
         """
@@ -795,10 +866,10 @@ class Terminal(object):
 
         :rtype: ParameterizingString
         """
-        if not self.does_styling:
-            return NullCallableString()
-        return ParameterizingString(self._background_color,
-                                    self.normal, 'on_color')
+        if self.does_styling:
+            return ParameterizingString(self._background_color, self.normal, 'on_color')
+
+        return NullCallableString()
 
     def on_color_rgb(self, red, green, blue):
         """
@@ -835,6 +906,7 @@ class Terminal(object):
         inadvertently returning another terminal capability.
         """
         formatters = split_compound(value)
+        # Pylint sometimes thinks formatters isn't a list  # pylint: disable=not-an-iterable
         if all((fmt in COLORS or fmt in COMPOUNDABLES) for fmt in formatters):
             return getattr(self, value)
 
@@ -928,15 +1000,23 @@ class Terminal(object):
 
         Common return values are 0, 8, 16, 256, or 1 << 24.
 
-        This may be used to test whether the terminal supports colors,
-        and at what depth, if that's a concern.
+        This may be used to test whether the terminal supports colors, and at what depth, if that's
+        a concern.
+
+        If this property is assigned a value of 88, the value 16 will be saved. This is due to the
+        the rarity of 88 color support and the inconsistency of behavior between implementations.
+
+        Assigning this property to a value other than 0, 4, 8, 16, 88, 256, or 1 << 24 will raise an
+        :py:exc:`AssertionError`.
         """
         return self._number_of_colors
 
     @number_of_colors.setter
     def number_of_colors(self, value):
-        assert value in (0, 4, 8, 16, 256, 1 << 24)
-        self._number_of_colors = value
+        assert value in (0, 4, 8, 16, 88, 256, 1 << 24)
+        # Because 88 colors is rare and we can't guarantee consistent behavior,
+        # when 88 colors is detected, it is treated as 16 colors
+        self._number_of_colors = 16 if value == 88 else value
         self.__clear_color_capabilities()
 
     @property
@@ -1024,6 +1104,22 @@ class Terminal(object):
             width = self.width
         return Sequence(text, self).center(width, fillchar)
 
+    def truncate(self, text, width=None):
+        r"""
+        Truncate ``text`` to maximum ``width`` printable characters, retaining terminal sequences.
+
+        :arg str text: Text to truncate
+        :arg int width: The maximum width to truncate it to
+        :rtype: str
+        :returns: ``text`` truncated to at most ``width`` printable characters
+
+        >>> term.truncate(u'xyz\x1b[0;3m', 2)
+        u'xy\x1b[0;3m'
+        """
+        if width is None:
+            width = self.width
+        return Sequence(text, self).truncate(width)
+
     def length(self, text):
         u"""
         Return printable length of a string containing sequences.
@@ -1129,7 +1225,7 @@ class Terminal(object):
         return result
 
     def wrap(self, text, width=None, **kwargs):
-        """
+        r"""
         Text-wrap a string, returning a list of wrapped lines.
 
         :arg str text: Unlike :func:`textwrap.wrap`, ``text`` may contain
@@ -1138,7 +1234,7 @@ class Terminal(object):
             :func:`string.expandtabs`.
         :arg int width: Unlike :func:`textwrap.wrap`, ``width`` will
             default to the width of the attached terminal.
-        :arg kwargs: See :py:class:`textwrap.TextWrapper`
+        :arg \**kwargs: See :py:class:`textwrap.TextWrapper`
         :rtype: list
         :returns: List of wrapped lines
 
@@ -1149,9 +1245,7 @@ class Terminal(object):
         wrapper = SequenceTextWrapper(width=width, term=self, **kwargs)
         lines = []
         for line in text.splitlines():
-            lines.extend(
-                (_linewrap for _linewrap in wrapper.wrap(line))
-                if line.strip() else (u'',))
+            lines.extend(iter(wrapper.wrap(line)) if line.strip() else (u'',))
 
         return lines
 
@@ -1168,7 +1262,7 @@ class Terminal(object):
         the keyboard string at a time. This method should always return
         without blocking if called after :meth:`kbhit` has returned True.
 
-        Implementors of alternate input stream methods should override
+        Implementers of alternate input stream methods should override
         this method.
         """
         assert self._keyboard_fd is not None
@@ -1205,7 +1299,7 @@ class Terminal(object):
         ready_r = [None, ]
         check_r = [self._keyboard_fd] if self._keyboard_fd is not None else []
 
-        while HAS_TTY and True:
+        while HAS_TTY:
             try:
                 ready_r, _, _ = select.select(check_r, [], [], timeout)
             except InterruptedError:
@@ -1269,6 +1363,7 @@ class Terminal(object):
             # Save current terminal mode:
             save_mode = termios.tcgetattr(self._keyboard_fd)
             save_line_buffered = self._line_buffered
+            # pylint: disable-next=possibly-used-before-assignment
             tty.setcbreak(self._keyboard_fd, termios.TCSANOW)
             try:
                 self._line_buffered = False
@@ -1287,13 +1382,13 @@ class Terminal(object):
         r"""
         A context manager for :func:`tty.setraw`.
 
-        Although both :meth:`break` and :meth:`raw` modes allow each keystroke
+        Although both :meth:`cbreak` and :meth:`raw` modes allow each keystroke
         to be read immediately after it is pressed, Raw mode disables
         processing of input and output.
 
         In cbreak mode, special input characters such as ``^C`` or ``^S`` are
         interpreted by the terminal driver and excluded from the stdin stream.
-        In raw mode these values are receive by the :meth:`inkey` method.
+        In raw mode these values are received by the :meth:`inkey` method.
 
         Because output processing is not done, the newline ``'\n'`` is not
         enough, you must also print carriage return to ensure that the cursor
@@ -1345,8 +1440,8 @@ class Terminal(object):
             self.stream.write(self.rmkx)
             self.stream.flush()
 
-    def inkey(self, timeout=None, esc_delay=0.35):
-        """
+    def inkey(self, timeout=None, esc_delay=DEFAULT_ESCDELAY):
+        r"""
         Read and return the next keyboard event within given timeout.
 
         Generally, this should be used inside the :meth:`raw` context manager.
@@ -1354,12 +1449,16 @@ class Terminal(object):
         :arg float timeout: Number of seconds to wait for a keystroke before
             returning.  When ``None`` (default), this method may block
             indefinitely.
-        :arg float esc_delay: To distinguish between the keystroke of
-           ``KEY_ESCAPE``, and sequences beginning with escape, the parameter
-           ``esc_delay`` specifies the amount of time after receiving escape
-           (``chr(27)``) to seek for the completion of an application key
-           before returning a :class:`~.Keystroke` instance for
-           ``KEY_ESCAPE``.
+        :arg float esc_delay: Time in seconds to block after Escape key
+           is received to await another key sequence beginning with
+           escape such as *KEY_LEFT*, sequence ``'\x1b[D'``], before returning a
+           :class:`~.Keystroke` instance for ``KEY_ESCAPE``.
+
+           Users may override the default value of ``esc_delay`` in seconds,
+           using environment value of ``ESCDELAY`` as milliseconds, see
+           `ncurses(3)`_ section labeled *ESCDELAY* for details.  Setting
+           the value as an argument to this function will override any
+           such preference.
         :rtype: :class:`~.Keystroke`.
         :returns: :class:`~.Keystroke`, which may be empty (``u''``) if
            ``timeout`` is specified and keystroke is not received.
@@ -1367,11 +1466,16 @@ class Terminal(object):
         .. note:: When used without the context manager :meth:`cbreak`, or
             :meth:`raw`, :obj:`sys.__stdin__` remains line-buffered, and this
             function will block until the return key is pressed!
-        """
-        resolve = functools.partial(resolve_sequence,
-                                    mapper=self._keymap,
-                                    codes=self._keycodes)
 
+        .. note:: On Windows, a 10 ms sleep is added to the key press detection loop to reduce CPU
+            load. Due to the behavior of :py:func:`time.sleep` on Windows, this will actually
+            result in a 15.6 ms delay when using the default `time resolution
+            <https://docs.microsoft.com/en-us/windows/win32/api/timeapi/nf-timeapi-timebeginperiod>`_.
+            Decreasing the time resolution will reduce this to 10 ms, while increasing it, which
+            is rarely done, will have a perceptable impact on the behavior.
+
+        _`ncurses(3)`: https://www.man7.org/linux/man-pages/man3/ncurses.3x.html
+        """
         stime = time.time()
 
         # re-buffer previously received keystrokes,
@@ -1384,14 +1488,14 @@ class Terminal(object):
             ucs += self.getch()
 
         # decode keystroke, if any
-        ks = resolve(text=ucs)
+        ks = resolve_sequence(ucs, self._keymap, self._keycodes)
 
         # so long as the most immediately received or buffered keystroke is
         # incomplete, (which may be a multibyte encoding), block until until
         # one is received.
         while not ks and self.kbhit(timeout=_time_left(stime, timeout)):
             ucs += self.getch()
-            ks = resolve(text=ucs)
+            ks = resolve_sequence(ucs, self._keymap, self._keycodes)
 
         # handle escape key (KEY_ESCAPE) vs. escape sequence (like those
         # that begin with \x1b[ or \x1bO) up to esc_delay when
@@ -1406,10 +1510,10 @@ class Terminal(object):
         if ks.code == self.KEY_ESCAPE:
             esctime = time.time()
             while (ks.code == self.KEY_ESCAPE and
-                   ucs in self._keymap_prefixes and
+                   ucs in self._keymap_prefixes and  # pylint: disable=unsupported-membership-test
                    self.kbhit(timeout=_time_left(esctime, esc_delay))):
                 ucs += self.getch()
-                ks = resolve(text=ucs)
+                ks = resolve_sequence(ucs, self._keymap, self._keycodes)
 
         # buffer any remaining text received
         self.ungetch(ucs[len(ks):])
