@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+"""Tests for capturing keyboard input"""
+
 # std imports
 import os
 import sys
@@ -8,11 +10,10 @@ import signal
 import platform
 
 # 3rd party
-import six
-import mock
 import pytest
 
 # local
+from blessed._compat import PY2, StringIO
 from .accessories import (SEMAPHORE,
                           RECV_SEMAPHORE,
                           SEND_SEMAPHORE,
@@ -22,15 +23,25 @@ from .accessories import (SEMAPHORE,
                           read_until_eof,
                           read_until_semaphore,
                           init_subproc_coverage)
+from .conftest import IS_WINDOWS, TEST_KEYBOARD, TEST_QUICK, TEST_RAW
+
+try:
+    from unittest import mock
+except ImportError:
+    import mock
+
+got_sigwinch = False
 
 pytestmark = pytest.mark.skipif(
-    os.environ.get('TEST_KEYBOARD', None) != 'yes' or platform.system() == 'Windows',
+    not TEST_KEYBOARD or IS_WINDOWS,
     reason="Timing-sensitive tests please do not run on build farms.")
 
-@pytest.mark.skipif(os.environ.get('TEST_QUICK', None) is not None,
-                    reason="TEST_QUICK specified")
+
+@pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_kbhit_interrupted():
     """kbhit() should not be interrupted with a signal handler."""
+    # pylint: disable=global-statement
+
     import pty
     pid, master_fd = pty.fork()
     if pid == 0:
@@ -71,10 +82,11 @@ def test_kbhit_interrupted():
     assert math.floor(time.time() - stime) == 1.0
 
 
-@pytest.mark.skipif(os.environ.get('TEST_QUICK', None) is not None,
-                    reason="TEST_QUICK specified")
+@pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_kbhit_interrupted_nonetype():
     """kbhit() should also allow interruption with timeout of None."""
+    # pylint: disable=global-statement
+
     import pty
     pid, master_fd = pty.fork()
     if pid == 0:
@@ -124,7 +136,7 @@ def test_kbhit_no_kb():
     """kbhit() always immediately returns False without a keyboard."""
     @as_subprocess
     def child():
-        term = TestTerminal(stream=six.StringIO())
+        term = TestTerminal(stream=StringIO())
         stime = time.time()
         assert term._keyboard_fd is None
         assert not term.kbhit(timeout=1.1)
@@ -134,11 +146,10 @@ def test_kbhit_no_kb():
 
 def test_kbhit_no_tty():
     """kbhit() returns False immediately if HAS_TTY is False"""
-    import blessed.terminal
     @as_subprocess
     def child():
         with mock.patch('blessed.terminal.HAS_TTY', False):
-            term = TestTerminal(stream=six.StringIO())
+            term = TestTerminal(stream=StringIO())
             stime = time.time()
             assert term.kbhit(timeout=1.1) is False
             assert math.floor(time.time() - stime) == 0
@@ -162,7 +173,7 @@ def test_keystroke_0s_cbreak_noinput_nokb():
     """0-second keystroke without data in input stream and no keyboard/tty."""
     @as_subprocess
     def child():
-        term = TestTerminal(stream=six.StringIO())
+        term = TestTerminal(stream=StringIO())
         with term.cbreak():
             stime = time.time()
             inp = term.inkey(timeout=0)
@@ -171,8 +182,7 @@ def test_keystroke_0s_cbreak_noinput_nokb():
     child()
 
 
-@pytest.mark.skipif(os.environ.get('TEST_QUICK', None) is not None,
-                    reason="TEST_QUICK specified")
+@pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_keystroke_1s_cbreak_noinput():
     """1-second keystroke without input; '' should be returned after ~1 second."""
     @as_subprocess
@@ -186,13 +196,12 @@ def test_keystroke_1s_cbreak_noinput():
     child()
 
 
-@pytest.mark.skipif(os.environ.get('TEST_QUICK', None) is not None,
-                    reason="TEST_QUICK specified")
+@pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_keystroke_1s_cbreak_noinput_nokb():
     """1-second keystroke without input or keyboard."""
     @as_subprocess
     def child():
-        term = TestTerminal(stream=six.StringIO())
+        term = TestTerminal(stream=StringIO())
         with term.cbreak():
             stime = time.time()
             inp = term.inkey(timeout=1)
@@ -302,7 +311,7 @@ def test_keystroke_0s_cbreak_multibyte_utf8():
     assert math.floor(time.time() - stime) == 0.0
 
 
-# Avylove: Added delay which should account for race contition. Re-add skip if randomly fail
+# Avylove: Added delay which should account for race condition. Re-add skip if randomly fail
 # @pytest.mark.skipif(os.environ.get('TRAVIS', None) is not None,
 #                     reason="travis-ci does not handle ^C very well.")
 @pytest.mark.skipif(platform.system() == 'Darwin',
@@ -367,8 +376,7 @@ def test_keystroke_0s_cbreak_sequence():
     assert math.floor(time.time() - stime) == 0.0
 
 
-@pytest.mark.skipif(os.environ.get('TEST_QUICK', None) is not None,
-                    reason="TEST_QUICK specified")
+@pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_keystroke_1s_cbreak_with_input():
     """1-second keystroke w/multibyte sequence; should return after ~1 second."""
     import pty
@@ -399,8 +407,7 @@ def test_keystroke_1s_cbreak_with_input():
     assert math.floor(time.time() - stime) == 1.0
 
 
-@pytest.mark.skipif(os.environ.get('TEST_QUICK', None) is not None,
-                    reason="TEST_QUICK specified")
+@pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_esc_delay_cbreak_035():
     """esc_delay will cause a single ESC (\\x1b) to delay for 0.35."""
     import pty
@@ -434,8 +441,7 @@ def test_esc_delay_cbreak_035():
     assert 34 <= int(duration_ms) <= 45, duration_ms
 
 
-@pytest.mark.skipif(os.environ.get('TEST_QUICK', None) is not None,
-                    reason="TEST_QUICK specified")
+@pytest.mark.skipif(TEST_QUICK, reason="TEST_QUICK specified")
 def test_esc_delay_cbreak_135():
     """esc_delay=1.35 will cause a single ESC (\\x1b) to delay for 1.35."""
     import pty
@@ -576,7 +582,7 @@ def test_get_location_0s():
     """0-second get_location call without response."""
     @as_subprocess
     def child():
-        term = TestTerminal(stream=six.StringIO())
+        term = TestTerminal(stream=StringIO())
         stime = time.time()
         y, x = term.get_location(timeout=0)
         assert (math.floor(time.time() - stime) == 0.0)
@@ -587,11 +593,11 @@ def test_get_location_0s():
 # jquast: having trouble with these tests intermittently locking up on Mac OS X 10.15.1,
 # that they *lock up* is troublesome, I tried to use "pytest-timeout" but this conflicts
 # with our retry module, so, just skip them entirely.
-@pytest.mark.skipif(not os.environ.get('TEST_RAW'), reason="TEST_RAW not specified")
+@pytest.mark.skipif(not TEST_RAW, reason="TEST_RAW not specified")
 def test_get_location_0s_under_raw():
     """0-second get_location call without response under raw mode."""
     import pty
-    pid, master_fd = pty.fork()
+    pid, _ = pty.fork()
     if pid == 0:
         cov = init_subproc_coverage('test_get_location_0s_under_raw')
         term = TestTerminal()
@@ -612,11 +618,11 @@ def test_get_location_0s_under_raw():
     assert math.floor(time.time() - stime) == 0.0
 
 
-@pytest.mark.skipif(not os.environ.get('TEST_RAW'), reason="TEST_RAW not specified")
+@pytest.mark.skipif(not TEST_RAW, reason="TEST_RAW not specified")
 def test_get_location_0s_reply_via_ungetch_under_raw():
     """0-second get_location call with response under raw mode."""
     import pty
-    pid, master_fd = pty.fork()
+    pid, _ = pty.fork()
     if pid == 0:
         cov = init_subproc_coverage('test_get_location_0s_reply_via_ungetch_under_raw')
         term = TestTerminal()
@@ -644,7 +650,7 @@ def test_get_location_0s_reply_via_ungetch():
     """0-second get_location call with response."""
     @as_subprocess
     def child():
-        term = TestTerminal(stream=six.StringIO())
+        term = TestTerminal(stream=StringIO())
         stime = time.time()
         # monkey patch in an invalid response !
         term.ungetch(u'\x1b[10;10R')
@@ -658,9 +664,10 @@ def test_get_location_0s_reply_via_ungetch():
 def test_get_location_0s_nonstandard_u6():
     """u6 without %i should not be decremented."""
     from blessed.formatters import ParameterizingString
+
     @as_subprocess
     def child():
-        term = TestTerminal(stream=six.StringIO())
+        term = TestTerminal(stream=StringIO())
 
         stime = time.time()
         # monkey patch in an invalid response !
@@ -678,12 +685,12 @@ def test_get_location_styling_indifferent():
     """Ensure get_location() behavior is the same regardless of styling"""
     @as_subprocess
     def child():
-        term = TestTerminal(stream=six.StringIO(), force_styling=True)
+        term = TestTerminal(stream=StringIO(), force_styling=True)
         term.ungetch(u'\x1b[10;10R')
         y, x = term.get_location(timeout=0.01)
         assert (y, x) == (9, 9)
 
-        term = TestTerminal(stream=six.StringIO(), force_styling=False)
+        term = TestTerminal(stream=StringIO(), force_styling=False)
         term.ungetch(u'\x1b[10;10R')
         y, x = term.get_location(timeout=0.01)
         assert (y, x) == (9, 9)
@@ -694,7 +701,7 @@ def test_get_location_timeout():
     """0-second get_location call with response."""
     @as_subprocess
     def child():
-        term = TestTerminal(stream=six.StringIO())
+        term = TestTerminal(stream=StringIO())
         stime = time.time()
         # monkey patch in an invalid response !
         term.ungetch(u'\x1b[0n')
@@ -705,11 +712,95 @@ def test_get_location_timeout():
     child()
 
 
-@pytest.mark.skipif(six.PY2, reason="Python 3 only")
+def test_get_fgcolor_0s():
+    """0-second get_fgcolor call without response."""
+    @as_subprocess
+    def child():
+        term = TestTerminal(stream=StringIO())
+        stime = time.time()
+        rgb = term.get_fgcolor(timeout=0)
+        assert math.floor(time.time() - stime) == 0.0
+        assert rgb == (-1, -1, -1)
+    child()
+
+
+def test_get_fgcolor_0s_reply_via_ungetch():
+    """0-second get_fgcolor call with response."""
+    @as_subprocess
+    def child():
+        term = TestTerminal(stream=StringIO())
+        stime = time.time()
+        term.ungetch(u'\x1b]10;rgb:a0/52/2d\x07')  # sienna
+
+        rgb = term.get_fgcolor(timeout=0.01)
+        assert math.floor(time.time() - stime) == 0.0
+        assert rgb == (160, 82, 45)
+    child()
+
+
+def test_get_fgcolor_styling_indifferent():
+    """Ensure get_fgcolor() behavior is the same regardless of styling"""
+    @as_subprocess
+    def child():
+        term = TestTerminal(stream=StringIO(), force_styling=True)
+        term.ungetch(u'\x1b]10;rgb:d2/b4/8c\x07')  # tan
+        rgb = term.get_fgcolor(timeout=0.01)
+        assert rgb == (210, 180, 140)
+
+        term = TestTerminal(stream=StringIO(), force_styling=False)
+        term.ungetch(u'\x1b]10;rgb:40/e0/d0\x07')  # turquoise
+        rgb = term.get_fgcolor(timeout=0.01)
+        assert rgb == (64, 224, 208)
+    child()
+
+
+def test_get_bgcolor_0s():
+    """0-second get_bgcolor call without response."""
+    @as_subprocess
+    def child():
+        term = TestTerminal(stream=StringIO())
+        stime = time.time()
+        rgb = term.get_bgcolor(timeout=0)
+        assert math.floor(time.time() - stime) == 0.0
+        assert rgb == (-1, -1, -1)
+    child()
+
+
+def test_get_bgcolor_0s_reply_via_ungetch():
+    """0-second get_bgcolor call with response."""
+    @as_subprocess
+    def child():
+        term = TestTerminal(stream=StringIO())
+        stime = time.time()
+        term.ungetch(u'\x1b]11;rgb:99/32/cc\x07')  # darkorchid
+
+        rgb = term.get_bgcolor(timeout=0.01)
+        assert math.floor(time.time() - stime) == 0.0
+        assert rgb == (153, 50, 204)
+    child()
+
+
+def test_get_bgcolor_styling_indifferent():
+    """Ensure get_bgcolor() behavior is the same regardless of styling"""
+    @as_subprocess
+    def child():
+        term = TestTerminal(stream=StringIO(), force_styling=True)
+        term.ungetch(u'\x1b]11;rgb:ff/e4/c4\x07')  # bisque
+        rgb = term.get_bgcolor(timeout=0.01)
+        assert rgb == (255, 228, 196)
+
+        term = TestTerminal(stream=StringIO(), force_styling=False)
+        term.ungetch(u'\x1b]11;rgb:de/b8/87\x07')  # burlywood
+        rgb = term.get_bgcolor(timeout=0.01)
+        assert rgb == (222, 184, 135)
+    child()
+
+
+@pytest.mark.skipif(PY2, reason="Python 3 only")
 def test_detached_stdout():
     """Ensure detached __stdout__ does not raise an exception"""
     import pty
-    pid, master_fd = pty.fork()
+    pid, _ = pty.fork()
     if pid == 0:
         cov = init_subproc_coverage('test_detached_stdout')
         sys.__stdout__.detach()
