@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+"""Tests for Terminal methods that account for sequences in strings"""
+
 # std imports
 import os
 import sys
@@ -7,11 +9,12 @@ import platform
 import itertools
 
 # 3rd party
-import six
 import pytest
 
 # local
+from blessed._compat import StringIO, PY2
 from .accessories import TestTerminal, as_subprocess
+from .conftest import IS_WINDOWS
 
 if platform.system() != 'Windows':
     import fcntl
@@ -19,6 +22,7 @@ if platform.system() != 'Windows':
 
 
 def test_length_cjk():
+    """Test length of East Asian characters"""
     @as_subprocess
     def child():
         term = TestTerminal()
@@ -33,15 +37,36 @@ def test_length_cjk():
     child()
 
 
+def test_length_with_zwj_is_wrong():
+    """Because of the way Zero-Width Joiner (ZWJ) is measured, blessed gets this wrong"""
+    # But for the time being, so do many terminals (~85%), so its not a huge deal..
+    # https://ucs-detect.readthedocs.io/results.html
+    @as_subprocess
+    def child():
+        term = TestTerminal()
+        # RGI_Emoji_ZWJ_Sequence  ; family: woman, woman, girl, boy
+        given = term.bold_red(u'\U0001F469\u200D\U0001F469\u200D\U0001F467\u200D\U0001F466')
+        expected = sum((2, 0, 2, 0, 2, 0, 2))
+
+        # exercise,
+        assert term.length(given) == expected
+
+
 def test_length_ansiart():
+    """Test length of ANSI art"""
     @as_subprocess
     def child(kind):
-        import codecs
+        if PY2:
+            from codecs import open as open_
+        else:
+            open_ = open
+
         term = TestTerminal(kind=kind)
         # this 'ansi' art contributed by xzip!impure for another project,
         # unlike most CP-437 DOS ansi art, this is actually utf-8 encoded.
         fname = os.path.join(os.path.dirname(__file__), 'wall.ans')
-        lines = codecs.open(fname, 'r', 'utf-8').readlines()
+        with open_(fname, 'r', encoding='utf-8') as ansiart:
+            lines = ansiart.readlines()
         assert term.length(lines[0]) == 67  # ^[[64C^[[34m▄▓▄
         assert term.length(lines[1]) == 75
         assert term.length(lines[2]) == 78
@@ -49,14 +74,13 @@ def test_length_ansiart():
         assert term.length(lines[4]) == 78
         assert term.length(lines[5]) == 78
         assert term.length(lines[6]) == 77
-    kind = 'xterm-256color'
-    if platform.system() == 'Windows':
-        kind = 'vtwin10'
+    kind = 'vtwin10' if IS_WINDOWS else 'xterm-256color'
     child(kind)
 
 
 def test_sequence_length(all_terms):
-    """Ensure T.length(string containing sequence) is correcterm."""
+    """Ensure T.length(string containing sequence) is correct."""
+    # pylint: disable=too-complex,too-many-statements
     @as_subprocess
     def child(kind):
         term = TestTerminal(kind=kind, force_styling=True)
@@ -287,7 +311,7 @@ def test_sequence_length(all_terms):
         assert (term.length(term.cud(10)) == 0)
 
         # this is how manpages perform underlining, this is done
-        # with the 'overstrike' capability of teletypes, and aparently
+        # with the 'overstrike' capability of teletypes, and apparently
         # less(1), '123' -> '1\b_2\b_3\b_'
         text_wseqs = u''.join(itertools.chain(
             *zip(plain_text, itertools.cycle(['\b_']))))
@@ -303,7 +327,7 @@ def test_env_winsize():
         # set the pty's virtual window size
         os.environ['COLUMNS'] = '99'
         os.environ['LINES'] = '11'
-        term = TestTerminal(stream=six.StringIO())
+        term = TestTerminal(stream=StringIO())
         save_init = term._init_descriptor
         save_stdout = sys.__stdout__
         try:
@@ -321,14 +345,16 @@ def test_env_winsize():
     child()
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires fcntl")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires fcntl")
 def test_winsize(many_lines, many_columns):
     """Test height and width is appropriately queried in a pty."""
     pixel_width, pixel_height = 1024, 768
+
     @as_subprocess
     def child(lines=25, cols=80):
         # set the pty's virtual window size
         val = struct.pack('HHHH', lines, cols, pixel_width, pixel_height)
+        # pylint: disable-next=possibly-used-before-assignment
         fcntl.ioctl(sys.__stdout__.fileno(), termios.TIOCSWINSZ, val)
         term = TestTerminal()
         winsize = term._height_and_width()
@@ -343,14 +369,14 @@ def test_winsize(many_lines, many_columns):
 
 
 def test_Sequence_alignment_fixed_width(all_terms):
+    """Test alignment methods with width provided"""
     @as_subprocess
     def child(kind):
         term = TestTerminal(kind=kind)
         pony_msg = 'pony express, all aboard, choo, choo!'
         pony_len = len(pony_msg)
-        pony_colored = u''.join(
-            ['%s%s' % (term.color(n % 7), ch,)
-             for n, ch in enumerate(pony_msg)])
+        pony_colored = u''.join('%s%s' % (term.color(n % 7), ch,)
+                                for n, ch in enumerate(pony_msg))
         pony_colored += term.normal
         ladjusted = term.ljust(pony_colored, 88)
         radjusted = term.rjust(pony_colored, 88)
@@ -366,7 +392,7 @@ def test_Sequence_alignment_fixed_width(all_terms):
     child(kind=all_terms)
 
 
-@pytest.mark.skipif(platform.system() == 'Windows', reason="requires fcntl")
+@pytest.mark.skipif(IS_WINDOWS, reason="requires fcntl")
 def test_Sequence_alignment(all_terms):
     """Tests methods related to Sequence class, namely ljust, rjust, center."""
     @as_subprocess
@@ -378,9 +404,8 @@ def test_Sequence_alignment(all_terms):
 
         pony_msg = 'pony express, all aboard, choo, choo!'
         pony_len = len(pony_msg)
-        pony_colored = u''.join(
-            ['%s%s' % (term.color(n % 7), ch,)
-             for n, ch in enumerate(pony_msg)])
+        pony_colored = u''.join('%s%s' % (term.color(n % 7), ch,)
+                                for n, ch in enumerate(pony_msg))
         pony_colored += term.normal
         ladjusted = term.ljust(pony_colored)
         radjusted = term.rjust(pony_colored)
@@ -461,12 +486,12 @@ def test_sequence_is_movement_false(all_terms):
     def child(kind):
         from blessed.sequences import measure_length
         term = TestTerminal(kind=kind)
-        assert (0 == measure_length(u'', term))
+        assert measure_length(u'', term) == 0
         # not even a mbs
-        assert (0 == measure_length(u'xyzzy', term))
+        assert measure_length(u'xyzzy', term) == 0
         # negative numbers, though printable as %d, do not result
         # in movement; just garbage. Also not a valid sequence.
-        assert (0 == measure_length(term.cuf(-333), term))
+        assert measure_length(term.cuf(-333), term) == 0
         assert (len(term.clear_eol) == measure_length(term.clear_eol, term))
         # various erases don't *move*
         assert (len(term.clear_bol) == measure_length(term.clear_bol, term))
@@ -493,10 +518,10 @@ def test_sequence_is_movement_false(all_terms):
     child(all_terms)
 
 
-def test_termcap_will_move_false(all_terms):
+def test_termcap_will_move_false(all_terms):  # pylint: disable=too-complex,too-many-branches
     """Test parser about sequences that do not move the cursor."""
     @as_subprocess
-    def child(kind):
+    def child(kind):  # pylint: disable=too-many-branches
         from blessed.sequences import iter_parse
         term = TestTerminal(kind=kind)
         if term.clear_eol:
